//=========================================================================
//
//	FILE:				CP_Date.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class for dealing with dates.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 266 $
//						Latest Checkout:  $Date: 2005-08-29 21:53:00 -0700 (Mon, 29 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Date
#define _H_CP_Date
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Object.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

const SInt16 CP_DefaultDateFormat 			= 0;
const SInt16 CP_AbbreviateLength			= 3;
const SInt16 CP_LongDateLength				= 10;

    /*! Controls how the date data will be formatted for output. */
enum CP_E_DateFormat
{
	CP_E_FullDateFormat = 1,			/*!< The date is shown in full as DayOfWeek, Month Day, Year. */
	CP_E_LongDateFormat,				/*!< The date is shown in long format as DayOfWeek, Month Day, Year. */
	CP_E_ShortDateFormat,				/*!< The date is shown in short format as DayOfWeek, Month Day, Year. */
	CP_E_LongDateFormat_NoDay,			/*!< The date is shown in long format as DayOfWeek, Month Day, Year. */
	CP_E_ShortDateFormat_NoDay			/*!< The date is shown in short format as Month Day, Year. */
};

    /*! Controls how the date data will be formatted upon input. */
enum CP_E_DateFormatInput
{
	CP_E_MMDDYY_DateFormat,		/*!< All of the below formats are controlled by the MMDDYY entries. */
	CP_E_DDMMYY_DateFormat,
	CP_E_YYMMDD_DateFormat,
	CP_E_MMDDYYYY_DateFormat,
	CP_E_DDMMYYYY_DateFormat,
	CP_E_YYYYMMDD_DateFormat
};

/*!
*	@class	CP_Date
*	@brief	A class for dealing with dates.
*	
*	The CP_Date class provides methods for performing math on dates. Returning a specific
*	part of the date, such as year, month, day.
*
*	There are also methods to return the date in a specific formatted way.
*/
class CP_EXPORT CP_Date : public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Date();
										CP_Date( const CP_Date& inDate );
										CP_Date( UInt32 inJulianDate );
										CP_Date(
												const CP_String& inDateString,
												CP_E_DateFormat inDateFormat = CP_E_FullDateFormat,
												CP_E_DateFormatInput inDateFormatIn = CP_E_MMDDYYYY_DateFormat );
	//! Destructor.
										~CP_Date();
	//@}
										
	//! @name Getters
	//@{
										/*!
										*	Returns the date in Julian format.
										*	
										*	@exception	none
										*	
										*	@return		UInt32			The date in Julian format.
										*/
			UInt32						GetJulian()										{ return( fJulian ); }

										/*!
										*	Returns the year of the date.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			The year portion of the date.
										*/
			SInt16						GetYear()										{ JulianToMDY(); return( fYear ); }

										/*!
										*	Returns the month of the date.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			The month portion of the date.
										*/
			SInt16						GetMonth()										{ JulianToMDY(); return( fMonth ); }

										/*!
										*	Returns the date of the date.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			The day portion of the date.
										*/
			SInt16						GetDay()										{ JulianToMDY(); return( fDay ); }

										/*!
										*	Returns the date of the week this date falls on.
										*	
										*	@exception	none
										*	
										*	@return		UInt16			The date of the week of this date.
										*/
			SInt16						GetDayOfWeek()									{ JulianToWday(); return( fDayOfWeek ); }
			CP_String					GetMonthString( bool inShortVersion = false );
	//@}
			
	//! @name Math
	//@{
			CP_Date						operator + ( const SInt32 inDays );
			CP_Date						operator - ( const SInt32 inDays );
			UInt32						operator - ( CP_Date& inDate );

			CP_Date&					operator += ( const SInt32 inDays );
			CP_Date&					operator -= ( const SInt32 inDays );

			CP_Date&					operator ++();
			CP_Date&					operator --();

			const CP_Date&				operator =( const CP_Date& inDate );
			const CP_Date&				operator =( const CP_String& inDateString );
			bool						operator >( const CP_Date& inDate );
			bool						operator >( const CP_String& inDateString );
			bool						operator <( const CP_Date& inDate );
			bool						operator <( const CP_String& inDateString );
			bool						operator >=( const CP_Date& inDate );
			bool						operator >=( const CP_String& inDateString );
			bool						operator <=( const CP_Date& inDate );
			bool						operator <=( const CP_String& inDateString );
			bool						operator ==( const CP_Date& inDate );
			bool						operator ==( const CP_String& inDateString );
			bool						operator !=( const CP_Date& inDate );
			bool						operator !=( const CP_String& inDateString );

			const CP_Date&				AddMonths( SInt16 inMonths );
			const CP_Date&				SubtractMonths( SInt16 inMonths );
			const CP_Date&				AddYears( SInt16 inYears );
			const CP_Date&				SubtractYears( SInt16 inYears );
			const CP_Date&				AddDays( SInt16 inDays );
			const CP_Date&				SubtractDays( SInt16 inDays );

			SInt16						YearsOld();
			SInt16						ElapsedDays( CP_Date& inStartDate );
	//@}

	//! @name Format
	//@{
			CP_String					Format( SInt16 inFormat = CP_DefaultDateFormat ) const;
	//@}
		
	//! @name Setters
	//@{
			void						SetToSystemDate();

										/*!
										*	Sets the year of this date.
										*	
										*	@param		inYear			The year of this date.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetYear( SInt16 inYear )						{ fYear = inYear; MDYToJulian(); }

										/*!
										*	Sets the month of this date.
										*	
										*	@param		inMonth			The month of this date.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetMonth( SInt16 inMonth )						{ fMonth = inMonth; MDYToJulian(); }

										/*!
										*	Sets the date of this date.
										*	
										*	@param		inDay			The day of this date.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetDay( SInt16 inDay )							{ fDay = inDay; MDYToJulian(); }

										/*!
										*	Sets the date of the week of this date.
										*	
										*	@param		inDayOfWeek		The date of the week of this date.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						SetDayOfWeek( SInt16 inDayOfWeek )				{ fDayOfWeek = inDayOfWeek; MDYToJulian(); }
	//@}

	//! @name Testers
	//@{
			bool						IsValid();
	//@}

	//! @name parsing
	//@{
			bool						ParseDateString(
												const CP_String& inDate,
												SInt16& inMonth,
												SInt16& inDay,
												SInt16& inYear );
	//@}

private:
	//! @name Utility
	//@{
			void						MDYToJulian();
			void						JulianToWday();
			void						JulianToMDY();

			void						AdjustDays();

			bool						ParseDateString( const CP_String& inDate );
	//@}

	// class data
protected:
								
	SInt16								fYear;					/*! Year of the date. */
	SInt16								fMonth;					/*! Month of the date. */
	SInt16								fDay;					/*! Day of the date. */
								
	SInt16								fDayOfWeek;				/*! Day of week. 1 = Sunday, 2 = Monday, etc... 7 = Saturday */

	UInt32								fJulian;				/*! Julian representation of the date. */

	CP_E_DateFormat						fCurrentDateFormat;		/*! Formatting style for output. */

	CP_E_DateFormatInput				fInputDateFormat;		/*! Formatting style for input of dates. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Date(
												UInt32 inMacSeconds,
												bool inMacOSTime );
										CP_Date( LongDateRec inDateRec );
	//@}

	//! @name Getters
	//@{
			void						GetDate( LongDateRec& inDateRec );
	//@}

	//! @name Setters
	//@{
			void						SetDate( LongDateRec inDateRec );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Date( SYSTEMTIME inSystemTime );
	//@}

	//! @name Getters
	//@{
			void						GetDate( SYSTEMTIME& inSystemTime );
	//@}

	//! @name Setters
	//@{
			void						SetDate( SYSTEMTIME inSystemTime );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Date



