//=========================================================================
//
//	FILE:				CP_Font.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage an OSs native font.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2004-12-23 22:35:15 -0800 (Thu, 23 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Font
#define _H_CP_Font
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_M_ReferenceCounted.h"

	// STL
#include "list"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_WIN32

class CP_FontRecord;

typedef std::list<CP_FontRecord *> 	CP_FontRecordList;

//!  CP_FontRecord
/*!
  Class that maintains a mapping between Windows HFONTs/LOGFONTs and a CP_Font.
*/
/*!
*	@class		CP_FontRecord
*	@brief		Class that maintains a mapping between Windows HFONTs/LOGFONTs and a CP_Font.
*
*				In order to prevent excessive duplication of GDI resources (creating HFONTs for
*				identical LOGFONTs), manage a table of refcounted CP_FontRecords (which contain
*				a LOGFONT and HFONT). Instead of CP_Font creating a font, it calls
*				CP_FontRecord::LookupFont and if the identical font has already been created,
*				it will use the existing one.  HFONT disposal happens at the right time
*				as long as the CP_Font unselects the HFONT when it's finished using it.
*/
class CP_EXPORT CP_FontRecord : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
	#if UNICODE
										CP_FontRecord( LOGFONTW &inLogFont );
	#else
										CP_FontRecord( LOGFONTA &inLogFont );
	#endif
	//! Destructor.
										~CP_FontRecord();
	//@}

	//! @name Lookup
	//@{
	#if UNICODE
		static CP_FontRecord			*LookupFont(LOGFONTW &inLogFont);
	#else
		static CP_FontRecord			*LookupFont( LOGFONTA &inLogFont );
	#endif
	//@}

public:
	HFONT								fWinFont;		/*! Windows HFONT. */

	#if UNICODE
		LOGFONTW						fLogFontW;		/*! Windows LOGFONT structure containing information about the font. */
	#else
		LOGFONTA						fLogFontA;		/*! Windows LOGFONT structure containing information about the font.  */
	#endif

	static CP_FontRecordList			gFontList;		/*! Global or static list of font records. */
};
#endif

/*!
*	@class		CP_Font
*	@brief		A class to manage an OSs native font.
*/
class CP_EXPORT CP_Font : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Font( CP_Canvas& inOwnerCanvas );
	//! Destructor.
	virtual								~CP_Font() throw();
	//@}
			
	//! @name OwnerCanvas
	//@{
										/*!
										*	Returns the CP_Canvas object that this font is being used in.
										*	
										*	@exception	none
										*
										*	@return		CP_Canvas			The canvas using this font.
										*/
			CP_Canvas					GetOwnerCanvas()								{ return( fOwnerCanvas ); }
	//@}

	//! @name Save / Restore
	//@{
			void						Save();
			void						Restore();
	//@}
			
	//! @name Getters
	//@{
			SInt16						Height();
			SInt16						AscentHeight();
			
			CP_String					GetName();
			SInt16						GetPointSize();
			SInt16						GetFace();
	//@}
			
	//! @name Setters
	//@{
			void						SetName( const CP_String& inFontName );
			void						SetSize( const SInt16 inFontSize );
			void						SetFace( const SInt16 inFontFace );
			void						SetNameAndSize(
												const CP_String& inFontName,
												SInt16 inFontSize );

			void						SetFromFontSpec( CP_FontSpec& inFontSpec );

			void						SetCanvas( CP_Canvas& inCanvas );
			void						UseInCanvas( CP_Canvas& inCanvas );
			void						ResetCanvas();

			void						SetDefault();
	//@}

protected:
	//! @name Update
	//@{
			void						UpdateFont();
	//@}

	// class data
protected:
	CP_Canvas							fOwnerCanvas;			/*! CP_Canvas font is used in. */
	
	CP_String							fFontName;				/*! Name of the font. */
	SInt16								fFontSize;				/*! Size of the font, measured in points. */
	SInt16								fFontFace;				/*! Face for the font, normal, bold, italic, underline, etc. */
	
	#if TARGET_OS_MAC
		SInt16							fSavedFont;				/*! Saved font from the canvas. Used in Save/Restore to temporarily hold font info.*/
		SInt16							fSavedSize;				/*! Saved font size from the canvas. Used in Save/Restore to temporarily hold font info.*/
		SInt16							fSavedMode;				/*! Saved font drawing mode from the canvas. Used in Save/Restore to temporarily hold font info.*/
		SInt16							fSavedFace;				/*! Saved font face from the canvas. Used in Save/Restore to temporarily hold font info.*/

		SInt16							fFontNumber;			/*! MacOS font number for this font.*/

		SInt16							fSavedFontNumber;		/*! MacOS font number for this font saved version.*/
		SInt16							fSavedFontFace;			/*! MacOS face for this font saved version.*/
	#endif	// TARGET_OS_MAC

	#if TARGET_OS_WIN32
		HFONT							fWinFont;				/*! Windows HFONT info about the font.*/

		HFONT							fSavedFont;				/*! Windows HFONT info used to hold a saved font.*/

		HWND							fUpdateWindow;			/*! Window for the font.*/

		CP_FontRecord					*fFontRecord;			/*! Structure to main a link between HFONTS and CP_Fonts.*/

		static bool						gIgnoreSystemSizing;	/*! Flag to indicate how system font sizing is handled.*/

		#if UNICODE
			LOGFONTW					fLogFontW;				/*! Windows LOGFONT structure. */
		#endif
		
		LOGFONTA						fLogFontA;				/*! Windows LOGFONT structure. */
	#endif	// TARGET_OS_WIN32

	#if TARGET_OS_LINUX
		GdkFont							*fGDKFont;				/*! GdkFont used to represent this object under Linux Gdk. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
public:
	//! @name Windows font structures
	//@{
										/*!
										*	Returns the Windows HFONT for this font.
										*	
										*	@exception	none
										*
										*	@return		HFONT			The Windows HFONT for this font.
										*/
			HFONT						GetWinFontObject()						{ return( fWinFont ); }

			void						WrapFont( HFONT inFont );

	#if UNICODE
			void						WrapLogFont( LOGFONTW *inLogFont );

										/*!
										*	Returns the Windows LOGFONT structure for this font.
										*	
										*	@exception	none
										*
										*	@return		LOGFONT			The Windows LOGFONT for this font.
										*/
			LOGFONTW					GetLogFont()							{ return( fLogFontW ); }
	#else
			void						WrapLogFont( LOGFONTA *inLogFont );

										/*!
										*	Returns the Windows LOGFONT structure for this font.
										*	
										*	@exception	none
										*
										*	@return		LOGFONT			The Windows LOGFONT for this font.
										*/
			LOGFONTA					GetLogFont()							{ return( fLogFontA ); }
	#endif

										/*!
										*	Sets a flag indicating that the Windows HWND using this font needs
										*	to be updated.
										*
										*	@param		inHwnd		The window needing to be updated.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetUpdateWindow( HWND inHwnd )			{ fUpdateWindow = inHwnd; }

										/*!
										*	Sets a flag to indicate that system sizing is to be ignored.
										*
										*	@param		b			State of the ignore system sizing flag.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
			static void					SetIgnoreSystemSizing( bool b = true )	{  gIgnoreSystemSizing = b; }

private:
	static	SInt16 APIENTRY				FindFontProc(
												LPLOGFONT lpLogFont,
												LPTEXTMETRIC lpTEXTMETRICs,
												DWORD fFontType,
												LPINT lpData );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Getters
	//@{
			GdkFont						*GetGdkFont();
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Font

