//=========================================================================
//
//	FILE:				CP_Rect.h
//	
//						CopyfRight  2003. Ken Stahlman. All fRights Reserved.
//	
//	PURPOSE:			Class for handling a Rect measurement using floating
//						point coordinates.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 227 $
//						Latest Checkout:  $Date: 2005-07-03 20:04:35 -0700 (Sun, 03 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Rect
#define _H_CP_Rect
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Point.h"
#include "CP_Size.h"

#ifdef __CF_USE_FRAMEWORK_INCLUDES__
	#include <Carbon/Carbon.h>				// Carbon framework
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_Rect
*	@brief	A class for representing a rect using floating point coordinates.
*	
*	This class is used to express all rectangles in CPLAT.
*/
class CP_EXPORT CP_Rect
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Rect();
										CP_Rect(
												float infLeftCoord,
												float infTopCoord,
												float infRightCoord,
												float infBottomCoord );
										CP_Rect(
												const CP_Point& inLocation,
												const CP_Size& inSize );
										CP_Rect( const CP_Rect& inRect );
	//! Destructor.
	virtual								~CP_Rect();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Return the fLeft coordinate.
										*	
										*	@exception	none
										*
										*	@return	float const		Left coordinate
										*/
			float						Left() const											{ return( fLeft ); }

										/*!
										*	Return the fTop coordinate.
										*	
										*	@exception	none
										*	
										*	@return	float const		Top coordinate
										*/
			float						Top() const												{ return( fTop ); }

										/*!
										*	Return the fRight coordinate.
										*	
										*	@exception	none
										*	
										*	@return	float const		Right coordinate
										*/
			float						Right() const											{ return( fRight ); }

										/*!
										*	Return the fBottom coordinate.
										*	
										*	@exception	none
										*	
										*	@return	float const		Bottom coordinate
										*/
			float						Bottom() const											{ return( fBottom ); }

										/*!
										*	Return the width of this rectangle.
										*	
										*	@exception	none
										*	
										*	@return	float const		Width of rectangle.
										*/
			float						Width() const											{ return( fRight - fLeft ); }

										/*!
										*	Return the height of this rectangle.
										*	
										*	@exception	none
										*	
										*	@return	float const		Height of rectangle.
										*/
			float						Height() const											{ return( fBottom - fTop ); }
			
			float						MinX() const;
			float						MaxX() const;
			float						MinY() const;
			float						MaxY() const;

			float						CenterX() const;
			float						CenterY() const;
			CP_Point					Center() const;
	//@}

	//! @name Operators
	//@{
			CP_Rect&					operator = ( const CP_Rect& inFloatRect );
	//@}

	//! @name Setters
	//@{
			void						Set(
												float infLeft,
												float infTop,
												float infRight,
												float infBottom );
			void						Set(
												const CP_Point& inTopLeft,
												const CP_Size& inSize );
			void						SetWidth( float inWidth );
			void						SetHeight( float inHeight );


										/*!
										*	Sets the left coordinates from the passed in float.
										*
										*	@param	inLeft			coordinate for the left side.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetLeft( float inLeft )									{ fLeft = inLeft; }

										/*!
										*	Sets the top coordinates from the passed in float.
										*
										*	@param	inTop			coordinate for the top side.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetTop( float inTop )									{ fTop = inTop; }

										/*!
										*	Sets the right coordinates from the passed in float.
										*
										*	@param	inRight			coordinate for the right side.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetRight( float inRight )								{ fRight = inRight; }

										/*!
										*	Sets the bottom coordinates from the passed in float.
										*
										*	@param	inBottom		coordinate for the bottom side.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						SetBottom( float inBottom )								{ fBottom = inBottom; }
	//@}

	//! @name Comparers
	//@{
			bool						operator == ( const CP_Rect& inRect ) const;
			bool						operator != ( const CP_Rect& inRect ) const;
	//@}

	//! @name Math
	//@{
			CP_Rect&					operator += ( CP_Rect& inRect );
			CP_Rect&					operator -= ( CP_Rect& inRect );
			CP_Rect&					operator *= ( CP_Rect& inRect );
			CP_Rect&					operator /= ( CP_Rect& inRect );

			CP_Rect						operator + ( CP_Rect& inRect ) const;
			CP_Rect						operator - ( CP_Rect& inRect ) const;
			CP_Rect						operator * ( CP_Rect& inRect ) const;
			CP_Rect						operator / ( CP_Rect& inRect ) const;

			CP_Rect&					operator += ( CP_Point& inPoint );
			CP_Rect&					operator -= ( CP_Point& inPoint );
			CP_Rect&					operator *= ( CP_Point& inPoint );
			CP_Rect&					operator /= ( CP_Point& inPoint );

			CP_Rect						operator + ( CP_Point& inPoint ) const;
			CP_Rect						operator - ( CP_Point& inPoint ) const;
			CP_Rect						operator * ( CP_Point& inPoint ) const;
			CP_Rect						operator / ( CP_Point& inPoint ) const;

	//! @name Movement
	//@{
			void						Offset(
												float horAmt,
												float verAmt );
			void						Offset( const CP_Point& amt );
	//@}
			
	//! @name Sizers
	//@{
			void						Expand(
												float horAmt,
												float verAmt );
			void						Expand( const CP_Rect& inAmount );

										/*!
										*	Expands this rect by the distance passed in the CP_Point. If
										*	the point contains negative numbers, then this rect is shrunk.
										*
										*	@param	inAmount			distance to expand this rect by.
										*	
										*	@exception	none
										*	
										*	@return	void
										*/
			void						Expand( const CP_Point& inAmount )						{ Expand( inAmount.h(), inAmount.v() ); }

			void						Inset(
											float horAmt,
											float verAmt );
			void						Inset( const CP_Rect& inAmount );
			void						Inset( const CP_Point& inAmount );
	//@}
			
	//! @name Testers
	//@{
			bool						IsEmpty() const;
			bool						IsValid() const;
			
			bool						Contains( const CP_Rect& inRect );
			bool						Contains( const CP_Point& inPoint );
	//@}

	//! @name Utility
	//@{
			void						MakeEmpty();
			void						MakeValid();
			
										/*!
										*	Returns the top left points of the rect as a CP_Point.
										*	
										*	@exception	none
										*
										*	@return	CP_Point			Contains the top left corner of this rect.
										*/
			CP_Point					TopLeft() const											{ return( CP_Point( fLeft, fTop ) ); }

										/*!
										*	Returns the bottom right points of the rect as a CP_Point.
										*	
										*	@exception	none
										*
										*	@return	CP_Point			Contains the bottom right corner of this rect.
										*/
			CP_Point					BottomRight() const										{ return( CP_Point( fRight, fBottom ) ); }

										/*!
										*	Returns the top right points of the rect as a CP_Point.
										*	
										*	@exception	none
										*
										*	@return	CP_Point			Contains the top right corner of this rect.
										*/
			CP_Point					TopRight() const										{ return( CP_Point( fRight, fTop ) ); }

										/*!
										*	Returns the bottom left points of the rect as a CP_Point.
										*	
										*	@exception	none
										*
										*	@return	CP_Point			Contains the bottom left corner of this rect.
										*/
			CP_Point					BottomLeft() const										{ return( CP_Point( fLeft, fBottom ) ); }

			CP_Size						Size() const											{ return( CP_Size( Width(), Height() ) ); }

			bool						Intersects( const CP_Rect& inRect );
			CP_Rect						Intersect( const CP_Rect& inRect ) const;
	//@}

protected:

	// class data
protected:
	float								fLeft;				/*! left coordinate. */
	float								fTop;				/*! top coordinate. */
	float								fRight;				/*! right coordinate. */
	float								fBottom;			/*! bottom coordinate. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	#if UNIVERSAL_INTERFACES_VERSION >= 0x0400
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Rect( HIRect inRect );
	//@}

	//! @name Converters
	//@{
										operator HIRect *();
										operator const HIRect *() const;
										operator const HIRect () const;

			CP_Rect&					operator = ( HIRect inRect );

			void						Set( HIRect inRect );
	//@}
	#endif	// UNIVERSAL_INTERFACES_VERSION 0x0400
	
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Rect( Rect inRect );
	//@}

	//! @name Converters
	//@{
										operator const Rect () const;
										operator Rect *();
										operator const Rect *() const;

			CP_Rect&					operator = ( Rect inRect );

			void						Set( Rect inRect );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
									CP_Rect( RECT *inRect );
	//@}

	//! @name Converters
	//@{
									operator RECT *();
									operator const RECT *() const;
									operator const RECT () const;

			void					Get( RECT& inOutRect );
			void					Set( RECT& inRect );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Rect

