//=========================================================================
//
//	FILE:				CP_BevelButton.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms bevel button control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 261 $
//						Latest Checkout:  $Date: 2005-08-27 20:20:55 -0700 (Sat, 27 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_BevelButton
#define _H_CP_BevelButton
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Menu.h"
#include "CP_OSIcon.h"
#include "CP_OSPicture.h"
#include "CP_PlatformControl.h"
#include "CP_Structure_Defines.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_MAC
typedef ControlBevelButtonBehavior OSButtonBehavior;
	enum
	{
		kOSButtonBehaviorPushbutton = kControlBehaviorPushbutton,
		kOSButtonBehaviorSticky		= kControlBehaviorSticky
	};	
#else
	typedef UInt16 OSButtonBehavior;
	enum
	{
		kOSButtonBehaviorPushbutton = 0,
		kOSButtonBehaviorSticky		= 0x0200
	};
	
	enum
	{
		kCP_E_MouseOverButton		= 1,
		kCP_E_ButtonFocused			= 2
	};	
#endif

const SInt16 kCP_BevelButtonSize						= 4;

/*!
*	@class		CP_BevelButton
*	@brief		Implementation of a platforms bevel button control.
*
*				If the platform does not support a native bevel button control,
*				then we have a custom implementation to implement it for the
*				underlying OS. Currently the bevel button is implemented on the
*				MacOS as a native control. Windows and Linux implement a version
*				that mimics the appearance and handling of a MacOS button.
*
*	@section 	persistance Persistance:
*				The following tags are implemented to create the button from
*				a persistance stream:
*					- kCP_Persist_ControlID - UInt32
*					- kCP_Persist_Bounds - CP_Rect
*					- kCP_Persist_Text - CP_String
*					- kCP_Persist_Command - UInt32
*					- kCP_Persist_IsHidden - bool
*					- kCP_Persist_IsEnabled - bool
*					- kCP_Persist_Thickness - SInt16
*					- kCP_Persist_MenuBehavior - SInt16
*					- kCP_Persist_MenuPlacement - SInt16
*					- kCP_Persist_Rounded - bool
*					- kCP_Persist_GraphicAlignment - SInt16
*					- kCP_Persist_GraphicXOffset - SInt16
*					- kCP_Persist_GraphicYOffset - SInt16
*					- kCP_Persist_TextPlacement - SInt16
*					- kCP_Persist_TextOffset - SInt16
*					- kCP_Persist_TextAlignment - SInt16
*					- kCP_Persist_SelectedItem - SInt16
*					- kCP_Persist_ContentType - SInt16
*					- kCP_Persist_Resourceid - SInt16
*
*	@section 	signals Signals and Slots:
*				The following signals and slot are implemented in this class:
*					- None
*
*				From CP_PlatformControl:
*					- kCP_Signal_Clicked
*					- kCP_Signal_ValueChanged
*
*	@ingroup	UI
*/
class CP_EXPORT CP_BevelButton : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_BevelButton();
										CP_BevelButton(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												const CP_S_ViewContentInfo& inContentInfo,
												UInt32 inCommand,
												OSButtonBehavior behavior = kOSButtonBehaviorPushbutton);
	//! Destructor.
	virtual								~CP_BevelButton() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Setters
	//@{
				void					SetContent( const CP_S_ViewContentInfo& inContentInfo );
				void					SetGraphicAlignment(
												GraphicAlignment inGraphicAlignment,
												SInt16 inHorOffset,
												SInt16 inVerOffset );
				void					SetTextAlignment(
												TextAlignment inTextAlignment,
												SInt16 inHorOffset );
				void					SetTextPlacement( TextPlacement inTextPlacement );
				void					SetMenu( SInt16 inMenuID );
				void					SetMenu( CP_Menu *inMenu );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );

			void						CommandUpdateStatus( CP_Event_Base& inEvent );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}


	// class data
protected:
	CP_S_ViewContentInfo				fContentInfo;
			
	CP_Menu								*fMenu;
	
	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		CP_String						fText;					/*! Text displayed on the button, if any. */
		GraphicAlignment				fGraphicAlignment;		/*! Alignment of the graphic, if any. */
		TextPlacement					fTextPlacement;			/*! Placement of the text relative to graphics. */
		TextAlignment					fTextAlignment;			/*! Alignment of the text */
		SInt16							fGraphicHorOffset;		/*! Horizontal offset of the graphic, if any. */
		SInt16							fGraphicVerOffset;		/*! Vertical offset of the graphic, if any. */
		SInt16							fTextOffset;			/*! Offset of the text, if any. */
	
		UInt32							fButtonDrawFlags;
		bool							fTrackLeave;
		bool							fMouseOverButton;		/*! Flag to indicate if the mouse has been over a button. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Events
	//@{
			void						HandleCommandProcessEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												UInt32 inCommand,
												OSButtonBehavior behavior);
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

protected:
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												UInt32 inCommand,
												OSButtonBehavior behavior);
	//@}

	//! @name Calculation
	//@{
			CP_Rect						CalcIconArea(
												CP_Canvas& inCanvas,
												CP_OSIcon& inIcon,
												const CP_Rect& inButtonBounds,
												GraphicAlignment inGraphicAlignment,
												SInt16 inHorOffset,
												SInt16 inVerOffset ) const;
			CP_Rect						CalcPictureArea(
												CP_Canvas& inCanvas,
												CP_OSPicture& inIcon,
												const CP_Rect& inButtonBounds,
												GraphicAlignment inGraphicAlignment,
												SInt16 inHorOffset,
												SInt16 inVerOffset ) const;
			CP_Rect						CalcTextArea(
												CP_Canvas& inCanvas,
												const CP_Rect& inButtonBounds,
												TextAlignment inTextAlignment );
			CP_Rect						CalcPopupArea();
	//@}

	//! @name Events
	//@{
			void						HandleMouseDownSlot( CP_Event_Base& inEvent );
			void						HandleMouseMovedSlot( CP_Event_Base& inEvent );
			void						HandleMouseLeaveSlot( CP_Event_Base& inEvent );

			void						DoPopupMenuClick();
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

	//! @name Events
	//@{
	virtual	void						HandleMouseDown( CP_Event_Base& inEvent );
	virtual	void						HandleMouseUp( CP_Event_Base& inEvent );
	//@}

protected:
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												UInt32 inCommand,
												OSButtonBehavior behavior );
	//@}

	//! @name Calculation
	//@{
			CP_Rect						CalcIconArea(
												CP_Canvas& inCanvas,
												CP_OSIcon& inIcon,
												const CP_Rect& inButtonBounds,
												GraphicAlignment inGraphicAlignment,
												SInt16 inHorOffset,
												SInt16 inVerOffset ) const;
			CP_Rect						CalcPictureArea(
												CP_Canvas& inCanvas,
												CP_OSPicture& inIcon,
												const CP_Rect& inButtonBounds,
												GraphicAlignment inGraphicAlignment,
												SInt16 inHorOffset,
												SInt16 inVerOffset ) const;
			CP_Rect						CalcTextArea(
												CP_Canvas& inCanvas,
												const CP_Rect& inButtonBounds,
												TextAlignment inTextAlignment );
			CP_Rect						CalcPopupArea();
	//@}

	//! @name Events
	//@{
			void						DoPopupMenuClick();
	//@}
#endif

};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_BevelButton
