//=========================================================================
//
//	FILE:				CP_OpenGLView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to draw its contents using OpenGL.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_OpenGLView
#define _H_CP_OpenGLView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueList.h"
#include "CP_OpenGL_Utilities.h"
#include "CP_View.h"

#if TARGET_OS_MAC
	#if TARGET_API_MAC_CARBON && __CF_USE_FRAMEWORK_INCLUDES__
		#include <agl/agl.h>
	#else
		#include "agl.h"
	#endif
#endif

#if TARGET_OS_WIN32
	typedef HGLRC				CP_GLContext;
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_OpenGLView
*	@brief		A view to display the views content using OpenGL methods.
*
*				The actual content of this view is drawn using OpenGL. In order
*				to not have to override this class for each view, the drawing code
*				calls upon a slot method, that you have previously connected to the
*				drawing signal of this class. This slot will be called whenever the
*				OpenGL drawing needs to be updated.
*
*				This code snipped shows the creation of the OpenGLView, and connecting
*				up the drawing slot. You can see this in the OpenGL demo.
*
*	@code
*					CPLAT::CP_Point					paneLocation( 10, 10 );
*					CPLAT::CP_Size					paneSize( 500 - 20, 300 - 20 );
*					CPLAT::CP_OpenGLView			*openGLView = NULL;
*	
*					openGLView = CP_new CPLAT::CP_OpenGLView( window, paneLocation, paneSize );
*					openGLView->FindSignal( kCP_Signal_DrawOpenGLContent )->connect(
*						Wonderlib::slot(this, &OpenGL_Application::DrawGLContent) );
*	@endcode
*
*				The slot that is called is defined as shown here. This is the location where
*				your drawing code would be performed.
* 
*	@code
*					void OpenGL_Application::DrawGLContent( CPLAT::CP_DataValueHolder& inValue,
*						CPLAT::OSStatus& inOutStatus )
*					{
*						glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
*						do_display();
*					}
*	@endcode
*
*	@section 	persistance Persistance:
*				The following tags are implemented to create the button from
*				a persistance stream:
*					- kCP_Persist_ControlID - UInt32
*					- kCP_Persist_Bounds - CP_Rect
*					- kCP_Persist_IsHidden - bool
*					- kCP_Persist_IsEnabled - bool
*
*	@section 	signals Signals and Slots:
*				The following signals and slot are implemented in this class:
*					- kCP_Signal_DrawOpenGLContent
*
*	@ingroup	UI
*/
class CP_EXPORT CP_OpenGLView : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_OpenGLView();
										CP_OpenGLView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_OpenGLView() throw();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Drawing
	//@{
			void						Redraw();
	//@}

protected:
	//! @name Drawing
	//@{
	virtual	void						DrawGL();
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						SwapBuffers();
	virtual	void						ReshapeGLView();
	//@}

	//! @name Frame
	//@{
	virtual	void						OnBoundsChanged(
												const CP_Rect& inPreviousBounds,
												const CP_Rect& inNewBounds );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
	//! @name Initialization
	//@{
			void						Init();
	//@}

	// class data
protected:
	#if TARGET_OS_MAC
		CP_AglContext					fAGLContext;			/* ! AglContext to draw into. */
	#endif
	
	#if TARGET_OS_WIN32
		CP_GLContext					fGLContext;				/* ! Windows OpenGL context to draw into. */
	#endif
	
	CP_DataValueSignal					fDrawSignal;			/* ! Signal called to perform the actual drawing. */
	
//========================================================================
// Platform specific routines
//========================================================================
#ifdef TARGET_OS_MAC
private:
			void						SetupAGL();
#endif

#if TARGET_OS_WIN32
protected:
	virtual	void						SetDCPixelFormat( HDC inHDC );

private:
			void						SetupGL();
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_OpenGLView

